@@ -29,38 +29,21 @@ __imlib_IsRealFile(const char *s)
 char               *
 __imlib_FileKey(const char *file)
 {
-   char               *newfile;
+   const char         *p;
 
-   newfile = malloc(strlen(file) + 1);
-   if (!newfile)
-      return NULL;
-   newfile[0] = 0;
-   {
-      char               *p1, *p2;
-      int                 go;
+   for (p = file;;)
+     {
+        p = strchr(p, ':');
+        if (!p)
+           break;
+        p++;
+        if (*p == '\0')
+           break;
+        if (*p != ':')          /* :: Drive spec? */
+           return strdup(p);
+        p++;
+     }
 
-      go = 0;
-      p1 = (char *)file;
-      p2 = newfile;
-      while (p1[0])
-        {
-           if (go)
-             {
-                p2[0] = p1[0];
-                p2++;
-             }
-           if ((p1[0] == ':') && (p1[1] != ':'))
-              go = 1;
-           if ((p1[0] == ':') && (p1[1] == ':'))
-              p1++;
-           p1++;
-        }
-      p2[0] = p1[0];
-   }
-   if (newfile[0])
-      return newfile;
-   else
-      free(newfile);
    return NULL;
 }
 
@@ -198,22 +181,6 @@ __imlib_FileModDateFd(int fd)
    return (st.st_mtime > st.st_ctime) ? st.st_mtime : st.st_ctime;
 }
 
-int
-__imlib_FileCanRead(const char *s)
-{
-   struct stat         st;
-
-   DP("%s: '%s'\n", __func__, s);
-
-   if (__imlib_FileStat(s, &st))
-      return 0;
-
-   if (!(st.st_mode & (S_IRUSR | S_IRGRP | S_IROTH)))
-      return 0;
-
-   return access(s, R_OK) == 0 ? 1 : 0; // ??? TBD
-}
-
 char              **
 __imlib_FileDir(const char *dir, int *num)
 {
@@ -224,26 +191,27 @@ __imlib_FileDir(const char *dir, int *num)
    struct dirent      *dp;
 
    if ((!dir) || (!*dir))
-      return 0;
+      return NULL;
+
    dirp = opendir(dir);
    if (!dirp)
-     {
-        *num = 0;
-        return NULL;
-     }
+      return NULL;
+
    /* count # of entries in dir (worst case) */
    for (dirlen = 0; readdir(dirp) != NULL; dirlen++)
       ;
    if (!dirlen)
      {
         closedir(dirp);
-        *num = dirlen;
         return NULL;
      }
-   names = (char **)malloc(dirlen * sizeof(char *));
 
+   names = malloc(dirlen * sizeof(char *));
    if (!names)
-      return NULL;
+     {
+        closedir(dirp);
+        return NULL;
+     }
 
    rewinddir(dirp);
    for (i = 0; i < dirlen;)
@@ -251,17 +219,19 @@ __imlib_FileDir(const char *dir, int *num)
         dp = readdir(dirp);
         if (!dp)
            break;
+
         if ((strcmp(dp->d_name, ".")) && (strcmp(dp->d_name, "..")))
           {
              names[i] = strdup(dp->d_name);
              i++;
           }
      }
+   closedir(dirp);
 
    if (i < dirlen)
       dirlen = i;               /* dir got shorter... */
-   closedir(dirp);
    *num = dirlen;
+
    /* do a simple bubble sort here to alphanumberic it */
    while (!done)
      {
@@ -279,6 +249,7 @@ __imlib_FileDir(const char *dir, int *num)
                }
           }
      }
+
    return names;
 }
 
@@ -349,3 +320,22 @@ __imlib_ItemInList(char **list, int size, char *item)
      }
    return 0;
 }
+
+#include <errno.h>
+
+FILE               *
+__imlib_FileOpen(const char *path, const char *mode)
+{
+   FILE               *fp;
+
+   for (;;)
+     {
+        fp = fopen(path, mode);
+        if (fp)
+           break;
+        if (errno != EINTR)
+           break;
+     }
+
+   return fp;
+}
