@@ -1,20 +1,11 @@
-#include "config.h"
+#include "common.h"
 
 #include <ft2build.h>
 #include FT_FREETYPE_H
 #include FT_GLYPH_H
-#include <math.h>
-#include <string.h>
-#include <sys/types.h>
 
-#include "blend.h"
-#include "colormod.h"
-#include "common.h"
 #include "file.h"
 #include "font.h"
-#include "image.h"
-#include "rgbadraw.h"
-#include "rotate.h"
 
 extern FT_Library   ft_lib;
 
@@ -216,6 +207,8 @@ __imlib_font_load(const char *name, int faceidx, int size)
 void
 __imlib_font_free(ImlibFont * fn)
 {
+   __imlib_font_remove_from_fallback_chain_imp(fn);
+
    fn->references--;
    if (fn->references == 0)
      {
