@@ -1,7 +1,7 @@
 #ifndef __SCALE
 #define __SCALE 1
 
-#include "common.h"
+#include "types.h"
 
 typedef struct _imlib_scale_info ImlibScaleInfo;
 
@@ -9,19 +9,20 @@ ImlibScaleInfo     *__imlib_CalcScaleInfo(ImlibImage * im,
                                           int sw, int sh,
                                           int dw, int dh, char aa);
 ImlibScaleInfo     *__imlib_FreeScaleInfo(ImlibScaleInfo * isi);
-void                __imlib_ScaleSampleRGBA(ImlibScaleInfo * isi, DATA32 * dest,
-                                            int dxx, int dyy, int dx, int dy,
-                                            int dw, int dh, int dow);
-void                __imlib_ScaleAARGBA(ImlibScaleInfo * isi, DATA32 * dest,
+void                __imlib_ScaleSampleRGBA(ImlibScaleInfo * isi,
+                                            uint32_t * dest, int dxx, int dyy,
+                                            int dx, int dy, int dw, int dh,
+                                            int dow);
+void                __imlib_ScaleAARGBA(ImlibScaleInfo * isi, uint32_t * dest,
                                         int dxx, int dyy, int dx, int dy,
                                         int dw, int dh, int dow, int sow);
-void                __imlib_ScaleAARGB(ImlibScaleInfo * isi, DATA32 * dest,
-                                       int dxx, int dyy, int dx, int dy,
-                                       int dw, int dh, int dow, int sow);
+void                __imlib_ScaleAARGB(ImlibScaleInfo * isi, uint32_t * dest,
+                                       int dxx, int dyy, int dx, int dy, int dw,
+                                       int dh, int dow, int sow);
 
 #ifdef DO_MMX_ASM
 void                __imlib_Scale_mmx_AARGBA(ImlibScaleInfo * isi,
-                                             DATA32 * dest,
+                                             uint32_t * dest,
                                              int dxx, int dyy, int dx, int dy,
                                              int dw, int dh, int dow, int sow);
 #endif
