@@ -32,7 +32,7 @@ static const char  *const _formats[] = { "png" };
 
 #define mm_check(p) ((const char *)(p) <= (const char *)im->fi->fdata + im->fi->fsize)
 
-typedef struct {
+typedef struct __PACKED__ {
    uint32_t            len;
    union {
       uint32_t            type;
@@ -41,7 +41,7 @@ typedef struct {
 } png_chunk_hdr_t;
 
 /* IHDR */
-typedef struct {
+typedef struct __PACKED__ {
    uint32_t            w;       // Width
    uint32_t            h;       // Height
    uint8_t             depth;   // Bit depth  (1, 2, 4, 8, or 16)
@@ -54,13 +54,13 @@ typedef struct {
 #define _PNG_IHDR_SIZE	(4 + 4 + 13 + 4)
 
 /* acTL */
-typedef struct {
+typedef struct __PACKED__ {
    uint32_t            num_frames;      // Number of frames
    uint32_t            num_plays;       // Number of times to loop this APNG.  0 indicates infinite looping.
 } png_actl_t;
 
 /* fcTL */
-typedef struct {
+typedef struct __PACKED__ {
    uint32_t            frame;   // --   // Sequence number of the animation chunk, starting from 0
    uint32_t            w;       // --   // Width of the following frame
    uint32_t            h;       // --   // Height of the following frame
@@ -73,12 +73,12 @@ typedef struct {
 } png_fctl_t;
 
 /* fdAT */
-typedef struct {
+typedef struct __PACKED__ {
    uint32_t            frame;   // --   // Sequence number of the animation chunk, starting from 0
    uint8_t             data[];
 } png_fdat_t;
 
-typedef struct {
+typedef struct __PACKED__ {
    png_chunk_hdr_t     hdr;
    union {
       png_ihdr_t          ihdr;
@@ -244,7 +244,8 @@ row_callback(png_struct * png_ptr, png_byte * new_row,
            PNG_PASS_COLS(im->w, pass), PNG_PASS_ROWS(im->h, pass));
         y = y0 + dy * row_num;
 
-        sptr = (const uint32_t *)new_row;       /* Assuming aligned */
+        sptr = PCAST(const uint32_t *, new_row);        /* Assuming aligned */
+
         dptr = im->data + y * im->w;
         for (x = x0; x < im->w; x += dx)
           {
@@ -341,7 +342,8 @@ _load(ImlibImage * im, int load_data)
 
    for (ic = 0;; ic++, fptr += 8 + len + 4)
      {
-        chunk = (const png_chunk_t *)fptr;
+        chunk = PCAST(const png_chunk_t *, fptr);
+
         len = htonl(chunk->hdr.len);
         D("Scan %3d: %06lx: %6d: %.4s: ", ic,
           fptr - (unsigned char *)im->fi->fdata, len, chunk->hdr.name);
@@ -429,7 +431,8 @@ _load(ImlibImage * im, int load_data)
 
    for (ic = 0;; ic++, fptr += 8 + len + 4)
      {
-        chunk = (const png_chunk_t *)fptr;
+        chunk = PCAST(const png_chunk_t *, fptr);
+
         len = htonl(chunk->hdr.len);
         D("Chunk %3d: %06lx: %6d: %.4s: ", ic,
           fptr - (unsigned char *)im->fi->fdata, len, chunk->hdr.name);
@@ -588,40 +591,41 @@ _load(ImlibImage * im, int load_data)
    return rc;
 }
 
+typedef struct {
+   png_bytep           data;
+} misc_data_t;
+
 static int
 _save(ImlibImage * im)
 {
    int                 rc;
-   FILE               *f;
+   FILE               *f = im->fi->fp;
    png_structp         png_ptr;
    png_infop           info_ptr;
+   misc_data_t         misc;
    uint32_t           *ptr;
    int                 x, y, j, interlace;
-   png_bytep           row_ptr, data;
+   png_bytep           row_ptr;
    png_color_8         sig_bit;
    ImlibImageTag      *tag;
    int                 quality = 75, compression = 3;
    int                 pass, n_passes = 1;
    int                 has_alpha;
 
-   f = fopen(im->fi->name, "wb");
-   if (!f)
-      return LOAD_FAIL;
-
    rc = LOAD_FAIL;
    info_ptr = NULL;
-   data = NULL;
+   misc.data = NULL;
 
    png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
    if (!png_ptr)
-      goto quit;
+      return LOAD_FAIL;
 
    info_ptr = png_create_info_struct(png_ptr);
    if (!info_ptr)
       goto quit;
 
    if (setjmp(png_jmpbuf(png_ptr)))
-      goto quit;
+      QUIT_WITH_RC(LOAD_FAIL);
 
    /* check whether we should use interlacing */
    interlace = PNG_INTERLACE_NONE;
@@ -650,7 +654,7 @@ _save(ImlibImage * im)
         png_set_IHDR(png_ptr, info_ptr, im->w, im->h, 8, PNG_COLOR_TYPE_RGB,
                      interlace, PNG_COMPRESSION_TYPE_BASE,
                      PNG_FILTER_TYPE_BASE);
-        data = malloc(im->w * 3 * sizeof(png_byte));
+        misc.data = malloc(im->w * 3 * sizeof(png_byte));
      }
    sig_bit.red = 8;
    sig_bit.green = 8;
@@ -721,11 +725,11 @@ _save(ImlibImage * im)
                     {
                        uint32_t            pixel = ptr[x];
 
-                       data[j++] = PIXEL_R(pixel);
-                       data[j++] = PIXEL_G(pixel);
-                       data[j++] = PIXEL_B(pixel);
+                       misc.data[j++] = PIXEL_R(pixel);
+                       misc.data[j++] = PIXEL_G(pixel);
+                       misc.data[j++] = PIXEL_B(pixel);
                     }
-                  row_ptr = (png_bytep) data;
+                  row_ptr = misc.data;
                }
              png_write_rows(png_ptr, &row_ptr, 1);
 
@@ -739,15 +743,9 @@ _save(ImlibImage * im)
    rc = LOAD_SUCCESS;
 
  quit:
-   free(data);
+   free(misc.data);
    png_write_end(png_ptr, info_ptr);
-   png_destroy_write_struct(&png_ptr, (png_infopp) & info_ptr);
-   if (info_ptr)
-      png_destroy_info_struct(png_ptr, (png_infopp) & info_ptr);
-   if (png_ptr)
-      png_destroy_write_struct(&png_ptr, (png_infopp) NULL);
-
-   fclose(f);
+   png_destroy_write_struct(&png_ptr, &info_ptr);
 
    return rc;
 }
