@@ -8,7 +8,7 @@
 
 static const char  *const _formats[] = { "tiff", "tif" };
 
-#define DD(fmt...)  DC(0x80, fmt)
+#define DD(fmt...)  DC(DBG_PFX, 0x80, fmt)
 
 static struct {
    const unsigned char *data, *dptr;
@@ -466,7 +466,7 @@ _save(ImlibImage * im)
    int                 i;
    ImlibImageTag      *tag;
 
-   tif = TIFFOpen(im->fi->name, "w");
+   tif = TIFFFdOpen(fileno(im->fi->fp), im->fi->name, "w");
    if (!tif)
       return LOAD_FAIL;
 
